use strict;
use vars qw($VERSION %IRSSI);

use Irssi::TextUI;

$VERSION = "0.2";
%IRSSI = (
    authors     => "Alexander Wirt",
    contact     => "formorer\@formorer.de",
    name        => "apm",
    description => "Shows your battery status in your Statusbar",
    license     => "GNU Public License",
    url         => "http://www.formorer.de/code",
);


#
#apm.pl 
#	apm.pl is a small script for displaying your Battery Level in irssi.
#	Just load the script and do a /statusbar window add apm
#	and a small box [APM: XX] should be displayed this is only possible 
#	on Computers where /proc/apm is existing. If AC is displayed your
#	computer is plugged in. 
#
#	/set apm_refresh <sec> 	changes the refreshing time of the display
#
#
#	Changelog:
#
#	0.2 - Added apm_refresh and some documentation
#	0.1 - Initial Release





my ($refresh, $last_refresh, $refresh_tag) = (10);

#check for existenz of /proc/apm
exit unless (-r "/proc/apm");

sub get_apm {
	open(RC, "/proc/apm");
		my $line = <RC>;
	close RC;
	my ($ver1, $ver2, $sysstatus, $acstat, $chargstat, $batstatus, $prozent, $remain) = split(/\s/,$line);

	if ($acstat eq "0x01") { return "AC" } else { return "$prozent" }
}

sub apm {
	my ($item, $get_size_only) = @_;
	my $apm = get_apm();
	$item->default_handler($get_size_only, undef, "APM: $apm", 1 );
	}


sub set_apm {

	$refresh = Irssi::settings_get_int('apm_refresh');
	$refresh = 1 if $refresh < 1;
	return if $refresh == $last_refresh;
	$last_refresh = $refresh;
	Irssi::timeout_remove($refresh_tag) if $refresh_tag;
	$refresh_tag = Irssi::timeout_add($refresh*1000, 'refresh_apm', undef);

}


sub refresh_apm {
	Irssi::statusbar_items_redraw('apm');
}

Irssi::statusbar_item_register('apm', '{sb $0-}', 'apm');
Irssi::statusbars_recreate_items();

Irssi::settings_add_int('misc', 'apm_refresh', $refresh);

set_apm();
Irssi::signal_add('setup changed', 'set_apm');
