use 5.6.0;
use File::Basename;
use IPC::Open3;
use Irssi 20021107.0841;
use Irssi::TextUI;
use locale;
use re 'eval';
use strict;
use vars qw($VERSION %IRSSI);

$VERSION = '1.2.2';
%IRSSI = (
          authors     => 'Clemens Heidinger',
          changed     => 'Sun Dec 15 21:58:25 CET 2002',
          contact     => 'spoooky@quizparanoia.org',
          description => 'write like an idiot',
          license     => 'BSD',
          modules     => 'IPC::Open3 File::Basename',
          name        => 'dau',
          sbitems     => 'daumode',
          url         => 'http://scripts.quizparanoia.org/'
         );

print CLIENTCRAP "dau.pl $VERSION loaded. For help type \%9/dau --help\%9";

################################################################################
# Register settings
################################################################################

# Integer

# boolean
Irssi::settings_add_bool('misc', 'dau_figlet_print_font', '0');

# String
Irssi::settings_add_str('misc', 'dau_delimiter_string', ' ');
Irssi::settings_add_str('misc', 'dau_figlet_fontlist', 'mnemonic,term,ivrit');
Irssi::settings_add_str('misc', 'dau_figlet_fontpath', &def_dau_figlet_fontpath);
Irssi::settings_add_str('misc', 'dau_figlet_fontpolicy', 'allow');
Irssi::settings_add_str('misc', 'dau_figlet_path', &def_dau_figlet_path);
Irssi::settings_add_str('misc', 'dau_moron_eol_style', 'new');
Irssi::settings_add_str('misc', 'dau_moron_substitutions_permissions', '00');
Irssi::settings_add_str('misc', 'dau_random_options', '--moron,--words --mixedcase,--delimiter,--dots --moron,--color --uppercase,--leet,--stutter --moron,--reverse');
Irssi::settings_add_str('misc', 'dau_remote_channellist', '');
Irssi::settings_add_str('misc', 'dau_remote_channelpolicy', 'deny');
Irssi::settings_add_str('misc', 'dau_remote_deop_reply', 'you are on my shitlist now @ $nick');
Irssi::settings_add_str('misc', 'dau_remote_devoice_reply', 'you are on my shitlist now @ $nick');
Irssi::settings_add_str('misc', 'dau_remote_op_reply', 'thx 4 op @ $nick');
Irssi::settings_add_str('misc', 'dau_remote_permissions', '00000');
Irssi::settings_add_str('misc', 'dau_remote_question_reply', 'alles klar @ $nick');
Irssi::settings_add_str('misc', 'dau_remote_voice_reply', 'thx 4 voice @ $nick');
Irssi::settings_add_str('misc', 'dau_standard_messages', 'hi @ all');
Irssi::settings_add_str('misc', 'dau_standard_options', '--random');
Irssi::settings_add_str('misc', 'dau_words_range', '1-4');

################################################################################
# Register signals
################################################################################

Irssi::signal_add_first('send text', \&signal_send_text);
Irssi::signal_add_last('command set', \&signal_command_set);
Irssi::signal_add_last('complete word', \&signal_complete_word);
Irssi::signal_add_last('event privmsg', \&signal_event_privmsg);
Irssi::signal_add_last('nick mode changed', \&signal_nick_mode_changed);

################################################################################
# Register statusbar items
################################################################################

Irssi::statusbar_item_register('daumode', undef, 'statusbar_daumode');
Irssi::statusbars_recreate_items();

################################################################################
# Register commands
################################################################################

Irssi::command_bind('dau', \&command_dau);

################################################################################
# Variables
################################################################################

my %option;
my %switches = (

               # These switches may be combined

               'combo'  => {
                            'chars'     => { 'sub' => \&switch_chars },
                            'color'     => {
                                            'sub'   => \&switch_color,
                                            'split' => {
                                                        'chars'  => 1,
                                                        'lines'  => 1,
                                                        'rchars' => 1,
                                                        'words'  => 1,
                                                       },
                                           },
                            'delimiter' => {
                                            'sub'    => \&switch_delimiter,
                                            'string' => { '*' => 1 },
                                           },
                            'dots'      => { 'sub' => \&switch_dots },
                            'figlet'    => { 'sub' => \&switch_figlet },
                            'me'        => { 'sub' => \&switch_me },
                            'moron'     => {
                                            'sub'   => \&switch_moron,
                                            'eol' => {
                                                       'classic'  => 1,
                                                       'new'      => 1,
                                                       },
                                            'perm' => {
                                                        '00'  => 1,
                                                        '01'  => 1,
                                                        '10'  => 1,
                                                        '11'  => 1,
                                                       },
                                           },
                            'leet'      => { 'sub' => \&switch_leet },
                            'mixedcase' => { 'sub' => \&switch_mixedcase },
                            'reverse'   => { 'sub' => \&switch_reverse },
                            'stutter'   => { 'sub' => \&switch_stutter },
                            'uppercase' => { 'sub' => \&switch_uppercase },
                            'words'     => { 'sub' => \&switch_words },
                           },

               # The following switches must not be combined

               'nocombo' => {
                             'changelog' => { 'sub' => \&switch_changelog },
                             'daumode'   => {
                                             'sub' => \&switch_daumode,
                                             'fix' => { '*' => 1 },
                                            },
                             'help'      => {
                                             'sub'     => \&switch_help,

                                             # setting changed/added => change/add it here

                                             'setting' => {
                                                           # Integer

                                                           # boolean
                                                           'dau_figlet_print_font' => 1,

                                                           # String
                                                           'dau_delimiter_string'                => 1,
                                                           'dau_figlet_fontlist'                 => 1,
                                                           'dau_figlet_fontpath'                 => 1,
                                                           'dau_figlet_fontpolicy'               => 1,
                                                           'dau_figlet_path'                     => 1,
                                                           'dau_moron_eol_style'                 => 1,
                                                           'dau_moron_substitutions_permissions' => 1,
                                                           'dau_random_options'                  => 1,
                                                           'dau_remote_channellist'              => 1,
                                                           'dau_remote_channelpolicy'            => 1,
                                                           'dau_remote_deop_reply'               => 1,
                                                           'dau_remote_devoice_reply'            => 1,
                                                           'dau_remote_op_reply'                 => 1,
                                                           'dau_remote_permissions'              => 1,
                                                           'dau_remote_question_reply'           => 1,
                                                           'dau_remote_voice_reply'              => 1,
                                                           'dau_standard_messages'               => 1,
                                                           'dau_standard_options'                => 1,
                                                           'dau_words_range'                     => 1,
                                                          },
                                            },
                             'random' => { 'sub' => \&switch_random },
                            },
);

################################################################################
# Code run once at start
################################################################################

set_settings();

# all available figlet fonts to %switches hash
while (<$option{dau_figlet_fontpath}/*.flf>) {
	my $font = (fileparse($_, qr/\..*/))[0];
	$switches{combo}{figlet}{font}{$font} = 1;
}

################################################################################
# Subroutines (commands)
################################################################################

sub command_dau {
	my ($data, $server, $witem) = @_;
	my $output;

	$output = parse_text($data);

	unless (defined($server) && $server && $server->{connected}) {
		$option{'print'} = 1;
	}
	unless ((defined($witem) && $witem &&
	       ($witem->{type} eq 'CHANNEL' || $witem->{type} eq 'QUERY')))
	{
		$option{'print'} = 1;
	}

	if ($option{daumode}) {

		if (defined($witem) && $witem &&
		   ($witem->{type} eq 'CHANNEL' || $witem->{type} eq 'QUERY'))
		{
			# daumode set with parameters

			if ($option{queue}{0}{daumode}{fix}) {
				$option{daumode_active}{$server->{tag}}{$witem->{name}} = 1;
				$option{daumode_active_fix}{$server->{tag}}{$witem->{name}} =
				$option{queue}{0}{daumode}{fix};

			# daumode set without parameters

			} elsif (!$option{daumode_active}{$server->{tag}}{$witem->{name}}) {
				$option{daumode_active}{$server->{tag}}{$witem->{name}} = 1;
				$option{daumode_active_fix}{$server->{tag}}{$witem->{name}} = '';

			# daumode unset

			} else {
				$option{daumode_active}{$server->{tag}}{$witem->{name}} = 0;
				$option{daumode_active_fix}{$server->{tag}}{$witem->{name}} = '';
			}

			Irssi::statusbar_items_redraw('daumode');
		}

		return;
	}

	# MSG (or CTCP ACTION) $output to active channel/query-window

	{
		no strict 'refs';
		output_text($witem, $witem->{name}, $output);
	}
}

################################################################################
# Subroutines (switches, must not be combined)
################################################################################

sub switch_changelog {
	my $output;

	$option{'print'} = 1;

	$output = &fix(<<'	END');
	CHANGELOG

	2002-05-05    release 0.1.0
	              initial release

	2002-05-06    release 0.1.1
	              bugfixes, minor changes

	2002-05-11    release 0.2.0
	              - new function: Put a single space after each character
	              - %9--moron%9: minor changes

	2002-05-12    release 0.3.0
	              new function: %9--mixedcase%9

	2002-05-17    release 0.4.0
	              function putting a single space after each character
	              changed name to %9--delimiter%9 and will accept any
	              delimiter string now (new setting for that)

	2002-05-20    release 0.4.1
	              some nice new substitutions for %9--moron%9

	2002-05-24    release 0.5.0
	              new settings controlling the behavior of %9--figlet%9

	2002-06-15    release 0.6.0
	              - new settings for %9--figlet%9
	              - %9--figlet%9 will omit lines only containing whitespace

	2002-06-16    release 0.6.1
	              bugfixes

	2002-06-16    release 0.6.2
	              bugfixes

	2002-06-17    release 0.7.0
	              - new settings for %9--moron%9
	              - new substitutions for %9--moron%9

	2002-06-19    release 0.8.0
	              new function: %9--dots%9
	              %9--moron%9: minor changes

	2002-06-23    release 0.9.0
	              - new: remote feature
	              - new substitutions for %9--moron%9
	              - %9--figlet%9: bugfixes

	2002-06-23    release 0.9.1
	              - %9--moron%9: minor changes
	              - remote feature: minor changes

	2002-06-29    release 0.9.2
	              new settings for the remote feature

	2002-07-23    release 0.9.3
	              new setting for %9--words%9

	2002-07-28    release 1.0.0
	              - Syntax changed
	              - Tabcompletion for the options
	              - Integrated help (%9--help%9)
	              - Integrated changelog (%9--changelog%9)
	              - Some of the options and settings have a different name
	                now
	              - %9--moron%9 no longer tears smilies apart
	              - new function: %9--leet%9
	              - new function: %9--reverse%9

	2002-07-28    release 1.0.1
	              bugfixes

	2002-09-01    release 1.0.2
	              - the script will now work properly even if the alias for
	                /SAY does not exist
	              - new substitutions for %9--moron%9

	2002-09-03    release 1.0.3
	              - bugfixes
	              - new option for %9--figlet%9: %9-font%9

	2002-09-03    release 1.0.4
	              bugfixes

	2002-09-03    release 1.0.5
	              bugfixes

	2002-09-09    release 1.1.0
	              - You can combine commands now!
	              - new substitutions for %9--moron%9
	              - bugfixes

	2002-11-22    release 1.2.0
	              - _A lot_ of rewriting
	              - Syntax for %9/dau%9's options changed. See %9/dau --help%9
	              - Some settings have changed their name and/or expect other
	                values.
	                Checkout %9/dau --help%9, %9/set dau_%9 and
	                %9/dau --help <setting>%9
	              - new option for %9--delimiter%9: %9-string%9
	              - new option for %9--moron%9: %9-eol%9
	              - new option for %9--moron%9: %9-perm%9
	              - new setting: %9dau_moron_eol_style%9
	              - new setting: %9dau_random_options%9
	              - new setting: %9dau_standard_messages%9
	              - new setting: %9dau_standard_options%9
	              - new remote features:
	                - new setting: %9dau_remote_question_reply%9
	                - new setting: %9dau_remote_voice_reply%9
	                - new setting: %9dau_remote_devoice_reply%9
	                - new setting: %9dau_remote_op_reply%9
	                - new setting: %9dau_remote_deop_reply%9
	              - new function: %9--color%9
	              - new function: %9--daumode%9
	              - new function: %9--random%9
	              - new function: %9--stutter%9
	              - new function: %9--uppercase%9
	              - new statusbar item: %9daumode%9

	2002-11-27    release 1.2.1
	              minor changes and one crash-fix

	2002-12-15    release 1.2.2
	              - Changing settings of 'hallo wie geht'-remote-feature
	                didn't become effective immediately
	              - new substitutions for %9--moron%9
	END

	return $output;
}

sub switch_daumode {
	$option{daumode} = 1;
}

sub switch_help {
	my $output;
	my $option_setting = $option{queue}{$option{counter}}{help}{setting};

	$option{'print'} = 1;

	if (!$option_setting) {
		$output = &fix(<<'		END');
		%9SYNOPSIS%9

		%9/dau [%Uoptions%U] [%Utext%9]

		%9DESCRIPTION%9

		Just write in the annoying way many lusers do. In contrast to
		many scripts of this kind, this wont do _anything_
		automatically unless you turn the feature on via an irssi
		setting or a command.

		%9OPTIONS%9

		%9--changelog%9
		     Print the scripts changelog

		%9--chars%9 %Utext
		     Only one character each line

		%9--color%9 [%9-split%9 %Uchars|lines|rchars|words%U] %Utext
		     Write in colors

		     %9-split%9:
		         %Uchars%U:  Every character another color
		         %Ulines%U:  Every line another color
		         %Urchars%U: Some characters one color
		         %Uwords%U:  Every word another color

		%9--daumode%9 [%9-fix%9 %Umodes%U]
		     Toggle daumode. If daumode is switched on, any future
		     message sent to channel will be daualized.

		     %9-fix%9:
		         Fix modes. %9/dau --daumode -fix '--moron --color'%9
		         f.e. will make you write like a color loving moron
		         in a channel all the time.

		     There is a statusbar item available displaying the current
		     status of the daumode. Add it with
		     %9/statusbar window add [-alignment <left|right>] daumode%9
		     You may customize the look of the statusbar item in the
		     theme file. f.e.:

		     sb_daumode = "{sb daumode $0$1-}";

		     # $0:  status of daumode (ON or OFF)
		     # $1-: fixed modes (set by %9-fix%9)

		     sb_daumode_fix = " [$*]";

		     # Special item formatting the modes fixed by switch %9-fix%9.
		     # If you don't want them to be displayed, set it to "".
		     # $* contains the string with these fixed modes.

		%9--delimiter%9 [%9-string%9 %Udelimiter string%U] %Utext
		     Insert a delimiter-string after each character

		     %9-string%9 %Ustring%U:
		         Override setting dau_delimiter_string. If this string
		         contains whitespace, you should quote the string with
		         single quotes.

		%9--dots%9 %Utext
		     Put some dots... after some words...

		%9--figlet%9 [%9-font%9 %Ufont%U] %Utext
		     Use figlet to write

		     %9-font%9 %Ufont%U:
		         The font to use

		%9--help%9 [%9-setting%9 %Usetting%U]
		     Show these lines

		     %9-setting%9 %Usetting%U:
		         More information about a specific setting

		%9--me%9
		     Send a CTCP ACTION instead of a PRIVMSG

		%9--mixedcase%9 %Utext
		     Write in mixed case

		%9--moron%9 [%9-eol %Ustring%U | %9-perm%9 %U[01][01]%U] %Utext
		     Write in uppercase, mix in some typos, perform some
		     substitutions on the text, ... Just write like a
		     moron

		     %9-eol%9 %Ustring%U:
		         Override setting dau_moron_eol_style

		     %9-perm%9 %U[01][01]%U:
		         Override setting dau_moron_substitutions_permissions

		%9--leet%9 %Utext
		     Write in leet speech

		%9--random%9 %Utext
		     Let dau.pl choose randomly options. Get these options from
		     the comma separated list of setting dau_random_options

		%9--reverse%9 %Utext
		     Reverse the input string

		%9--stutter%9 %Utext
		     Stutter a bit

		%9--uppercase%9 %Utext
		     Write in upper case

		%9--words%9 %Utext
		     Only a few words each line

		%9EXAMPLES%9

		%9/dau --uppercase --mixedcase %Ufoo bar baz%9
		     Will write %Ufoo bar baz%U in mixed case.
		     %Ufoo bar baz%U is sent _first_ to the uppercase
		     subroutine _then_ to mixedcase subroutine. The order you
		     specify the options on the commandline is important. You
		     can see what output a command produces without sending it
		     to the active channel/query-window by typing the command
		     out of a non-channel/query-window. There are thousands of
		     possiblilities combining the options, so some combinations
		     may produce a strange output. Try changing the order of
		     the options, that might help.

		%9/dau --color --figlet %Ufoo bar baz%9
		     %9--color%9 will insert colorcodes after some characters.
		     So the string will look like %U\003005f\003004o[...]%U when
		     leaving the color subroutine. %9--figlet%9 uses then that
		     string as its input. So you'll have finally an output like
		     %U005f004o[...]%U in the figlet latters.
		     So its better to use _first_ %9--figlet%9 _then_ %9--color%9.
		END
	}

	# setting changed/added => change/add them below

	# Integer

	# boolean

	elsif ($option_setting eq 'dau_figlet_print_font') {
		$output = &fix(<<'		END');
		%9dau_figlet_print_font%9 %Ubool

		Print a message which font will be used.
		END
	}

	# String

	elsif ($option_setting eq 'dau_delimiter_string') {
		$output = &fix(<<'		END');
		%9dau_delimiter_string%9 %Ustring

		Tell %9--delimiter%9 which delimiter to use.
		END
	}
	elsif ($option_setting eq 'dau_figlet_fontlist') {
		$output = &fix(<<'		END');
		%9dau_figlet_fontlist%9 %Ustring

		Comma separated list of fonts. Checkout
		%9/dau --help -setting dau_figlet_fontpolicy%9
		to see what this setting is good for. Use the program
		`showfigfonts` shipped with figlet to find these fonts.
		END
	}
	elsif ($option_setting eq 'dau_figlet_fontpath') {
		$output = &fix(<<'		END');
		%9dau_figlet_fontpath%9 %Ustring

		Path to the figlet-fonts (*.flf).
		END
	}
	elsif ($option_setting eq 'dau_figlet_fontpolicy') {
		$output = &fix(<<'		END');
		%9dau_figlet_fontpolicy%9 %Ustring

		Specifies the policy used to handle the fonts in
		dau_figlet_fontpath. If set to %Uallow%U, all fonts available
		will be used by the command. You can exclude some fonts by
		setting dau_figlet_fontlist. If set to %Udeny%U, no fonts but
		the ones listed in dau_figlet_fontlist will be used by the
		command. Useful if you have many annoying fonts in your
		fontpath and you want to permit only a few of them.
		END
	}
	elsif ($option_setting eq 'dau_figlet_path') {
		$output = &fix(<<'		END');
		%9dau_figlet_path%9 %Ustring

		Should point to the figlet-executable.
		END
	}
	elsif ($option_setting eq 'dau_moron_eol_style') {
		$output = &fix(<<'		END');
		%9dau_moron_eol_style%9 %Ustring

		What to do at End Of Line?

		%Uclassic%U: !!!??!!!!!????!??????????!!!1 (or similar)
		%Unew%U    : !!!??!!!!!????!??????????!!!1 (or similar) or
		         = and in a new line ?                      or
		         ??
		END
	}
	elsif ($option_setting eq 'dau_moron_substitutions_permissions') {
		$output = &fix(<<'		END');
		%9dau_moron_substitutions_permissions%9 %U[01][01]

		Controls whether %9--moron%9 should perform some
		substitutions on the text or not. These substitutions make
		only sense performed on german text.

		First Bit:
		    Perform First-Level-Substitutions?

		Second Bit:
		    Perform Second-Level-Substitutions?
		END
	}
	elsif ($option_setting eq 'dau_random_options') {
		$output = &fix(<<'		END');
		%9dau_random_options%9 %Ustring

		Comma separated list of options %9--random%9 will use. It will
		take randomly one item of the list. If you set it f.e. to
		%U--uppercase --color,--mixedcase%U,
		the probability of printing a colored, uppercased string hello
		will be 50% as well as the probabilty of printing a mixedcased
		string hello when typing %9/dau --random hello%9.
		END
	}
	elsif ($option_setting eq 'dau_remote_channellist') {
		$output = &fix(<<'		END');
		%9dau_remote_channellist%9 %Ustring

		Comma separated list of channels. Checkout
		%9/dau --help -setting dau_remote_channelpolicy%9
		for details.
		END
	}
	elsif ($option_setting eq 'dau_remote_channelpolicy') {
		$output = &fix(<<'		END');
		%9dau_remote_channelpolicy%9 %Ustring

		Using the default policy %Udeny%U the script won't do anything except in
		the channels listed in dau_remote_channellist. Using the policy
		%Uallow%U the script will reply to all channels but the ones listed in
		dau_remote_channellist.
		END
	}
	elsif ($option_setting eq 'dau_remote_deop_reply') {
		$output = &fix(<<'		END');
		%9dau_remote_deop_reply%9 %Ustring

		Message sent to channel if someone deops you (mode change -o).
		The string given will be processed by the same subroutine
		parsing the %9/dau%9 command.

		Special Variables:

		$nick: contains the nick of the one who changed the mode
		END
	}
	elsif ($option_setting eq 'dau_remote_devoice_reply') {
		$output = &fix(<<'		END');
		%9dau_remote_devoice_reply%9 %Ustring

		Message sent to channel if someone devoices you (mode change -v).
		The string given will be processed by the same subroutine
		parsing the %9/dau%9 command.

		Special Variables:

		$nick: contains the nick of the one who changed the mode
		END
	}
	elsif ($option_setting eq 'dau_remote_op_reply') {
		$output = &fix(<<'		END');
		%9dau_remote_op_reply%9 %Ustring

		Message sent to channel if someone ops you (mode change +o).
		The string given will be processed by the same subroutine
		parsing the %9/dau%9 command.

		Special Variables:

		$nick: contains the nick of the one who changed the mode
		END
	}
	elsif ($option_setting eq 'dau_remote_permissions') {
		$output = &fix(<<'		END');
		%9dau_remote_permissions%9 %U[01][01][01][01][01]

		Permit or forbid the remote features.

		First Bit:
		    Very, very useful feature. Will reply to the profound question
		    'hallo wie geht?' the appropriate answer.
		    Again, only useful for german users. ;-)

		Second Bit:
		    If someone gives you voice in a channel, thank him!

		Third Bit:
		    If someone gives you op in a channel, thank him!

		Fourth Bit:
		    If devoiced, print message

		Fifth Bit:
		    If deopped, print message
		END
	}
	elsif ($option_setting eq 'dau_remote_question_reply') {
		$output = &fix(<<'		END');
		%9dau_remote_question_reply%9 %Ustring

		Specify the reply string for the question 'hallo wie geht?'.
		The string given will be processed by the same subroutine
		parsing the %9/dau%9 command.

		Special Variables:

		$nick: contains the nick of the one who sent the message to which
		       dau.pl reacts
		END
	}
	elsif ($option_setting eq 'dau_remote_voice_reply') {
		$output = &fix(<<'		END');
		%9dau_remote_voice_reply%9 %Ustring

		Message sent to channel if someone voices you (mode change +v).
		The string given will be processed by the same subroutine
		parsing the %9/dau%9 command.

		Special Variables:

		$nick: contains the nick of the one who changed the mode
		END
	}
	elsif ($option_setting eq 'dau_standard_messages') {
		$output = &fix(<<'		END');
		%9dau_standard_messages%9 %Ustring

		Comma separated list of strings %9/dau%9 will use if the user
		omits the text on the commandline.
		END
	}
	elsif ($option_setting eq 'dau_standard_options') {
		$output = &fix(<<'		END');
		%9dau_standard_options%9 %Ustring

		Options %9/dau%9 will use if the user omits them on the commandline.
		END
	}
	elsif ($option_setting eq 'dau_words_range') {
		$output = &fix(<<'		END');
		%9dau_words_range%9 %Ui-j

		Setup the range howmany words the command should write per line.
		1 <= i <= j <= 9; i, j element { 1, ... , 9 }. If i == j the command
		will write i words to the active window.  Else it takes a random
		number k (element { i, ... , j }) and writes k words per
		line.
		END
	}

	return $output;
}

sub switch_random {
	my $data = shift;
	my $output;
	my (@options, $text);

	# Push each item of dau_random_options in the @options array.

	while ($option{dau_random_options} =~ /\s*([^,]+)\s*,?/g) {
		my $item = $1;
		push @options, $item;
	}

	# More than one item in @options. Choose one randomly but exclude
	# the last item chosen.

	if (@options > 1) {
		@options = grep { $_ ne $option{random_last} } @options;
		my $opt = @options[rand(@options)];
		$option{random_last} = $opt;
		$text .= $opt . ' ' . $data;
		$output = parse_text($text);
	}

	# Exact one item in @options - take that

	elsif (@options == 1) {
		my $opt = $options[0];
		$option{random_last} = $opt;
		$text .= $opt . ' ' . $data;
		$output = parse_text($text);
	}

	# No item in @options - call switch_moron()

	else {
		$output = &{ $switches{combo}{moron}{'sub'} }($output);
	}

	return $output;
}

################################################################################
# Subroutines (switches, may be combined)
################################################################################

sub switch_chars {
	my $data = shift;
	my $output;

	foreach my $char (split //, $data) {
		$output .= "$char\n";
	}
	return $output;
}

sub switch_color {
	my $data = shift;
	my ($output, $color, $split);
	my @colors = (0, 2 .. 13, 15);

	@colors = map { $_ = sprintf('%02d', $_) } @colors;

	if ($option{queue}{$option{counter}}{color}{split}) {
		$color = $option{queue}{$option{counter}}{color}{split};
	} else {
		$color = 'rchars';
	}

	if ($color eq 'chars') {
		$split = '';
	} elsif ($color eq 'lines') {
		$split = "\n";
	} elsif ($color eq 'words') {
		$split = '\s+';
	} else {
		$split = '.' x rand(10);
	}

	for (split /($split)/, $data) {
		if ($_ eq ',') {
			$output .= "\003" . $colors[rand(@colors)] . ',,';
		} elsif (/^\s*$/) {
			$output .= $_;
		} else {
			$output .= "\003" . $colors[rand(@colors)] . $_;
		}
	}

	return $output;
}

sub switch_delimiter {
	my $data = shift;
	my $output;
	my $option_delimiter_string;

	if ($option{queue}{$option{counter}}{delimiter}{string}) {
		$option_delimiter_string = $option{queue}{$option{counter}}{delimiter}{string};
	} else {
		$option_delimiter_string = $option{dau_delimiter_string};
	}

	foreach my $char (split //, $data) {
		$output .= $char . $option_delimiter_string;
	}
	return $output;
}

sub switch_dots {
	my $data = shift;

	$data =~ s/\s*[[:punct:]]*\s+(?!$)/
	                              if (rand(10) < 3) {
	                                  (rand(10) >= 5 ? ' ' : '')
	                                  .
	                                  ('...' . '.' x rand(5))
	                                  .
	                                  (rand(10) >= 5 ? ' ' : '')
	                              } else { ' ' }
	                             /xge;
	rand(10) >= 5 ? $data .= ' ' : 0;
	$data .= ('...' . '.' x rand(10));

	return $data;
}

sub switch_figlet {
	my $data = shift;
	my $skip = 1;
	my ($output, @fonts, %font, $font, @cache1, @cache2);

	unless (-e $option{dau_figlet_path}) {
		print_err('figlet not found. More information: %9/dau --help -setting dau_figlet_path%9');
		return;
	}

	if ($option{queue}{$option{counter}}{figlet}{font}) {
		$font = $option{queue}{$option{counter}}{figlet}{font};
	} else {
		while ($option{dau_figlet_fontlist} =~ /\s*([^,\s]+)\s*,?/g) {
			$font{$1} = 1;
		}
		foreach my $font (keys %{ $switches{combo}{figlet}{font} }) {
			if (lc($option{dau_figlet_fontpolicy}) eq 'allow') {
				push(@fonts, $font)
					unless ($font{$font});
			} elsif (lc($option{dau_figlet_fontpolicy}) eq 'deny') {
				push(@fonts, $font)
					if ($font{$font});
			} else {
				print_err('Invalid value for setting dau_figlet_fontpolicy. More information: %9/dau --help -setting dau_figlet_fontpolicy%9');
				return;
			}
		}
		if (@fonts == 0) {
			print_err('Cannot find figlet-fonts. Please check your figlet installation or dau.pl\'s settings for %9--figlet%9 (%9/set dau_figlet%9)');
			return;
		}
		$font = $fonts[rand(@fonts)];
	}

	my $childpid = open3(*IN, *OUT, *ERR, $option{dau_figlet_path}, '-f', $font, $data);
	my @errors = <ERR>;
	unless (@errors == 0) {
		print_err('Error during execution of figlet');
		return 1;
	}
	print_out("using figlet-font $font")
		if ($option{dau_figlet_print_font});
	while (<OUT>) {
		chomp;
		if (/^\s*$/ && $skip) {
			next;
		} else {
			$skip = 0;
		}
		push(@cache1, $_);
	}
	close OUT;
	waitpid($childpid, 0);
	$skip = 1;
	foreach (reverse @cache1) {
		chomp;
		if (/^\s*$/ && $skip) {
			next;
		} else {
			$skip = 0;
		}
		push(@cache2, $_);
	}
	foreach (reverse @cache2) {
		$output .= "$_\n";
	}

	return $output;
}

sub switch_leet {
	my $data = shift;

	$_ = $data;
	s'fucker'f@#$er'gi;
	s/hacker/h4x0r/gi;
	s/sucker/sux0r/gi;
	s/fear/ph34r/gi;
	s/dude/d00d/gi;
	s/rude/r00d/gi;
	s/\bthe\b/d4/gi;
	s/\byou\b/j00/gi;
	s/\bdo\b/d00/gi;
	s/\b(\w{3,})er\b/${1}0r/gi;
	tr/lLzZeEaAsSgGtTbBqQoOiIcC/11223344556677889900||((/;
	s/(\w)/rand(100) < 50 ? "\u$1" : "\l$1"/ge;

	return $_;
}

sub switch_me {
	my $data = shift;

	$option{command} = 'ACTION';

	return $data;
}

sub switch_mixedcase {
	my $data = shift;

	$data =~ s/([[:alpha:]])/rand(100) < 50 ? uc($1) : lc($1)/ge;

	return $data;
}

sub switch_moron {
	my $data = shift;
	my $output;
	my $option_eol_style;
	my $option_substitutions_permissions;
	local $" = '|';

	# Get values of settings/switchs

	if ($option{queue}{$option{counter}}{moron}{eol}) {
		$option_eol_style = $option{queue}{$option{counter}}{moron}{eol};
	} else {
		$option_eol_style = $option{dau_moron_eol_style};
	}

	if ($option{queue}{$option{counter}}{moron}{perm}) {
		$option_substitutions_permissions = $option{queue}{$option{counter}}{moron}{perm};
	} else {
		$option_substitutions_permissions = $option{dau_moron_substitutions_permissions};
	}

	# Generate list of nicks in current channel for later use

	my @nicks;
	my $window = Irssi::active_win();
	my $channel = $window->{active};

	if ($channel->{type} eq 'CHANNEL') {
		foreach my $nick ($channel->nicks()) {
			if ($window->{active_server}->{nick} ne $nick->{nick}) {
				push(@nicks, quotemeta($nick->{nick}));
			}
		}
	}

	$_ = $data;

	# remove puntuation marks at EOL and ensure there is a single space at EOL

	s/\s*[,;.:?!]*\s*$/ /;

	# Only whitespace? Remove it.

	s/^\s+$//;

	# Some substitutions making always sense ;-)

	eval { # Catch strange error
		s/^(@nicks): (.+)/$2@ $1 /;
	};

	s/\*g\*/'*ggg' . ('g' x rand(10)) . '*'/eg;

	if (int(rand(2))) {
		s/:(-)?\)/^^/g;
	} else {
		s/:(-)?\)/':' . $1 . ')))' . (')' x rand(10)) . ('9' x rand(4))/eg;
	}

	s/;(-)?\)/';' . $1 . ')))' . (')' x rand(10)) . ('9' x rand(4))/eg;
	s/:(-)?\(/':' . $1 . '(((' . ('(' x rand(10)) . ('8' x rand(4))/eg;
	s#:(-)?\/#':' . $1 . '///' . ('/' x rand(10)) . ('7' x rand(4))#eg;

	# Substitutions making sense performed on german text.
	# The user can turn them off/on the commandline or via an irssi setting

	if ($option_substitutions_permissions =~ /^1[01]$/) {
		s/\bist er\b/issa/gi;
		s/\bist\b/is/gi;

		s/\bda(?:ss|) du\b/dasu/gi;
		s/\bda(?:ss|)\b/das/gi;

		s/\bsag mal\b/samma/gi;
		s/\bnochmal\b/nomma/gi;
		s/mal\b/ma/gi;

		s/\bund nun\b/unnu/gi;
		s/\bnun\b/nu/gi;

		s/\bja\b/joh/gi;
		s/\bts\b/tzz/gi;
		s/\blol\b/ll/gi;
		s/\beine\b/ne/gi;
		s/\bends\b/enz/gi;
		s/\bnein\b/neh/gi;
		s/\bdenn\b/den/gi;
		s/\brofl\b/rfl/gi;
		s/\bniveau/niwo/gi;
		s/\bnicht\b/ned/gi;
		s/\bachso\b/axo/gi;
		s/\baber\b/abba/gi;
		s/\bdanke\b/thx/gi;
		s/\bbitte\b/plz/gi;
		s/\bjetzt\b/jez/gi;
		s/\bcool\b/kewl/gi;
		s/\bleute\b/ppl/gi;
		s/\bwider/wieder/gi;
		s/\bmachen\b/tun/gi;
		s/\bspiel\b/game/gi;
		s/\bfilm\b/movie/gi;
		s/\birgend/irgent/gi;
		s/\bmit dem\b/mim/gi;
		s/\bok(?:ay)?\b/k/gi;
		s/\brotfl\b/rtfl/gi;
		s/\bholen\b/hohlen/gi;
		s/\bfilme\b/moviez/gi;
		s/\bspiele\b/gamez/gi;
		s/\bweil du\b/weilu/gi;
		s/\bwenn du\b/wennu/gi;
		s/\broflol\b/rfll/gi;
		s/\blizenz\b/lizens/gi;
		s/\bstabil\b/stabiel/gi;
		s/\bsyntax\b/synthax/gi;
		s/\bprobleme\b/probs/gi;
		s/\bkein problem\b/np/gi;
		s/\bprogramm\b/proggie/gi;
		s/\boriginal\b/orginal/gi;
		s/\bstandard\b/standart/gi;
		s/\beinzige\b/einzigste/gi;
		s/\btolerant\b/tollerant/gi;
		s/\bprogramme\b/proggies/gi;
		s/\bfunktioniert\b/funzt/gi;
		s/\bfunktionieren\b/funzen/gi;
		s/\bviel gl(?:|ue)ck\b/gl/gi;
		s/\beigentlich\b/eigendlich/gi;
		s/\bdiskussion\b/disskusion/gi;
		s/^hallo\s*$/hallo wie geht /i;
		s/\bzumindest\b/zumindestens/gi;
		s/\bn(?:|ae)mlich\b/nhmlich/gi;
		s/\bdistribution\b/dystrubution/gi;
		s/\b(?:sorry|entschuldigung)\b/sry/gi;
		s/\b([[:lower:]]{2,})st du\b/${1}su/gi;
		s/\bdistributionen\b/dystrubutionen/gi;
		s/\b(?:kiste|box|b(?:|ue)chse)\b/byxe/gi;
		s/\bwa(h)?r/$1 eq 'h' ? 'war' : 'wahr'/egi;
		s/\bsei([dt])\b/$1 eq 'd' ? 'seit' : 'seid'/egi;
	}
	if ($option_substitutions_permissions =~ /^[01]1$/) {
		s/\b([[:lower:]]+[b-np-z])er\b/${1}a/gi;
		s/\b([fv])(?=[[:alpha:]]{2,})/lc($1) eq 'f' ? 'v' : 'f'/egi;
	}

	$data = $_;

	# Mix in some typos

	foreach my $word (split /([\s\n])/, $data) {
		unless ($data =~ /^hallo wie geht[\s?!1]*$/i ||
		        $word =~ /:(-)?\)/                    ||
		        $word =~ /;(-)?\)/                    ||
		        $word =~ /:(-)?\(/                    ||
		        $word =~ /:(-)?\//
		       )
		{
			if ((rand(100) <= 20) && length($word) > 1) {
				if ($word =~ s/([[:punct:]])$//) {
					my $random = rand(length($word));
					$random = 2
						if ($random == 1);
					(substr($word, $random, 1), substr($word, $random-1, 1)) =
					(substr($word, $random-1, 1), substr($word, $random, 1));
					$word .= $1;
				} else {
					my $random = rand(length($word));
					$random = 2
						if ($random == 1);
					(substr($word, $random, 1), substr($word, $random-1, 1)) =
					(substr($word, $random-1, 1), substr($word, $random, 1));
				}
			}
		}
		$output .= $word;
	}

	# 'plenken'

	$output =~ s/(\w+)([,;.:?!]+)/$1 $2/g;
	$output =~ s/([,;.:?!]+)(\w+)/$1 $2/g;

	# uppercase text

	$output = uc($output);

	# do something at EOL

	# 'classic' style

	if ($option_eol_style eq 'classic') {
		my @punct = qw(? !);
		$output .= $punct[rand(@punct)] x rand(5)
			for (1..15);

		if ($output =~ /\?$/) {
			$output .= "" x rand(10);
		} elsif ($output =~ /!$/) {
			$output .= "1" x rand(10);
		}
	}

	# 'new' style

	else {
		my $random = rand(100);

		# many punctuation marks at EOL

		if ($random <= 70) {
			my @punct = qw(? !);
			$output .= $punct[rand(@punct)] x rand(5)
				for (1..15);

			if ($output =~ /\?$/) {
				$output .= "" x rand(10);
			} elsif ($output =~ /!$/) {
				$output .= "1" x rand(10);
			}
		}

		# or '??' at EOL

		elsif ($random <= 85) {
			$output .= '??';
		}

		# or "=\n?" at EOL

		else {
			$output .= "=\n?";
		}
	}

	return $output;
}

sub switch_reverse {
	my $data = shift;

	$data = reverse($data);

	return $data;
}

sub switch_stutter {
	my $data = shift;
	my $output;
	my @words = qw(eeeh oeeeh aeeeh);

	foreach (split / /, $data) {
		if (rand(100) < 20) {
			$output .= ' ' . $words[rand(@words)] . ", $_";
		} else {
			$output .= ' ' . $_;
		}
	}

	$output =~ s/\s+/ /g;
	$output =~ s/\s*,\s+\@/ @/g;

	for (1 .. rand(length($output)/5)) {
		pos $output = rand(length($output));
		$output =~ s/\G\b([[:alpha:]]+)\b/$1, $1/;
	}
	for (1 .. rand(length($output)/10)) {
		pos $output = rand(length($output));
		$output =~ s/\G([[:alpha:]])/$1 . ($1 x rand(3))/e;
	}

	$output =~ s/^\s+//;

	return $output;
}

sub switch_uppercase {
	my $data = shift;

	$data = uc($data);

	return $data;
}

sub switch_words {
	my $data = shift;
	my $output;
	my @numbers;

	if ($option{dau_words_range} =~ /^([1-9])-([1-9])$/) {
		my $x = $1;
		my $y = $2;
		unless ($x <= $y) {
			print_err('Invalid value for setting dau_words_range. More information: %9/dau --help -setting dau_words_range%9');
			return;
		}
		if ($x == $y) {
			push(@numbers, $x);
		} elsif ($x < $y) {
			for (my $i = $x; $i <= $y; $i++) {
				push(@numbers, $i);
			}
		}
	} else {
		print_err('Invalid value for setting dau_words_range. More information: %9/dau --help -setting dau_words_range%9');
		return;
	}
	my $random = $numbers[rand(@numbers)];
	while ($data =~ /((?:.*?(?:\s+|$)){1,$random})/g) {
		$output .= "$1\n"
			unless (length($1) == 0);
		$random = $numbers[rand(@numbers)];
	}

	return $output;
}

################################################################################
# Subroutines (signals)
################################################################################

sub signal_command_set {

	# If setting dau_figlet_fontpath changes, generate a new list of the
	# figlet fonts

	my $args = shift;
	my $figlet_fontpath;

	if ($args =~ /^dau_figlet_fontpath\s*(.*)/) {
		$figlet_fontpath = $1;
		%{ $switches{combo}{figlet}{font} } = ();
	} else {
		return;
	}

	while (<$figlet_fontpath/*.flf>) {
		my $font = (fileparse($_, qr/\..*/))[0];
		$switches{combo}{figlet}{font}{$font} = 1;
	}
}

sub signal_complete_word {
	my ($list, $window, $word, $linestart, $want_space) = @_;

	# Parsing the commandline for dau.pl is relatively complicated.
	# TAB-Completion depends on commandline parsing in dau.pl.
	# Script autors looking for a simple example for irssi's
	# TAB-Completion are wrong here.

	my $server  = Irssi::active_server();
	my $channel = $window->{active};
	my @switches_combo   = map { $_ = "--$_" } keys %{ $switches{combo} };
	my @switches_nocombo = map { $_ = "--$_" } keys %{ $switches{nocombo} };
	my @nicks;

	# Only complete when the commandline starts with '/dau'.
	# If not, let irssi do the work

	return unless ($linestart =~ /^.dau/i);

	# Remove everything syntactically correct thing of $linestart.
	# If there is anything else but whitespace at the end of
	# commandline parsing, we have an syntax error.
	# If we have a syntax error, complete only nicks.

	$linestart =~ s/^.dau ?//i;

	# Generate list of nicks in current channel for later use

	if ($channel->{type} eq 'CHANNEL') {
		foreach my $nick ($channel->nicks()) {
			if ($nick->{nick} =~ /^\Q$word\E/i &&
			    $window->{active_server}->{nick} ne $nick->{nick})
			{
				push(@nicks, quotemeta($nick->{nick}));
			}
		}
	}

	# Variables

	my $combo = 0;                # Boolean: True if last switch was one of keys %{ $switches{combo} }
	my $syntax_error = 0;         # Boolean: True if syntax error found
	my $counter = 0;              # Integer: Counts First-Level-Options
	my $first_level_option = '';  # String:  Last First-Level-Option
	my $second_level_option = ''; # String:  Last Second-Level-Option
	my $third_level_option = 0;   # Boolean: True if found a Third-Level-Option

	# Parsing commandline now. Set variables accordingly.

	OUTER: while ($linestart =~ /^--(\w+) ?/g) {

		$second_level_option = '';
		$third_level_option  = 0;

		# Found a First-Level-Option (combo)

		if (ref($switches{combo}{$1}{'sub'})) {
			$first_level_option = $1;
			$combo = 1;
		}

		# Found a First-Level-Option (nocombo)

		elsif (ref($switches{nocombo}{$1}{'sub'}) && $counter == 0) {
			$first_level_option = $1;
			$combo = 0;
		}

		# Not a First-Level-Option => Syntax error

		else {
			$syntax_error = 1;
			last OUTER;
		}

		# Syntactically correct => remove it

		$linestart =~ s/^--\w+ ?//;

		# Checkout if there are Second- or Third-Level-Options

		INNER: while ($linestart =~ /^-(\w+)( ('?)((?{ $3 })[^']+'|\S+))? ?/g) {

			my $second_level = $1;
			my $quotes       = $3;
			my $third_level  = $4;

			# You see, the regexp above is not perfect, have to remove
			# the closing single quote if opening single quote found.
			# Evil workaround, but who cares...

			$third_level =~ s/'$// if ($quotes);

			# Do the same for combo and nocombo-options. They have to be
			# handled separately anyway.

			# combo...

			if ($combo) {

				# Found a Second-Level-Option

				if ($switches{combo}{$first_level_option}{$second_level}) {
					$second_level_option = $second_level;
				}

				# Not a Second-Level-Option => Syntax error

				else {
					$syntax_error = 1;
					last OUTER;
				}

				# Syntactically correct => remove it

				$linestart =~ s/^-\w+//;

				# Found something in the regexp of the INNER-while-loop-condition,
				# which is perhaps a Third-Level-Option

				if ($third_level) {

					# Found a Third-Level-Option

					if ($switches{combo}{$first_level_option}{$second_level_option}{$third_level} ||
                                            $switches{combo}{$first_level_option}{$second_level_option}{'*'})
					{
						$third_level_option = 1;

						# Syntactically correct => remove it

						$linestart =~ s/^( ('?)((?{ $2 })[^']+'|\S+))? ?//;
					}

					# Not a Third-Level-Option => Syntax error

					else {
						$syntax_error = 1;
						last OUTER;
					}

				# Nothing found which comes into question for a Third-Level-Option.
				# The commandline has to be empty now (remember: everything
				# syntactically correct has been removed) or we have a syntax error.

				} else {

					# Empty! Later we will complete to Third-Level-Options

					if ($linestart =~ /^\s*$/) {
						$third_level_option = 0;
					}

					# Not empty => Syntax error

					else {
						$syntax_error = 1;
						last OUTER;
					}
				}

			# nocombo...

			} else {

				# Found a Second-Level-Option

				if ($switches{nocombo}{$first_level_option}{$second_level}) {
					$second_level_option = $second_level;
				}

				# Not a Second-Level-Option => Syntax error

				else {
					$syntax_error = 1;
					last OUTER;
				}

				# Syntactically correct => remove it

				$linestart =~ s/^-\w+//;

				# Found something in the regexp of the INNER-while-loop-condition,
				# which is perhaps a Third-Level-Option

				if ($third_level) {

					# Found a Third-Level-Option

					if ($switches{nocombo}{$first_level_option}{$second_level_option}{$third_level} ||
                                            $switches{nocombo}{$first_level_option}{$second_level_option}{'*'})
					{
						$third_level_option = 1;

						# Syntactically correct => remove it

						$linestart =~ s/^( ('?)((?{ $2 })[^']+'|\S+))? ?//;
					}

					# Not a Third-Level-Option => Syntax error

					else {
						$syntax_error = 1;
						last OUTER;
					}

				# Nothing found which comes into question for a Third-Level-Option.
				# The commandline has to be empty now (remember: everything
				# syntactically correct has been removed) or we have a syntax error.

				} else {

					# Empty! Later we will complete to Third-Level-Options

					if ($linestart =~ /^\s*$/) {
						$third_level_option = 0;
					}

					# Not empty => Syntax error

					else {
						$syntax_error = 1;
						last OUTER;
					}
				}
			}
		}
	} continue {
		$counter++;
	}

	# End of commandline-parsing.
	# Everything syntactically correct removed.
	# If commandline is not empty now, we have a syntax error.

	if ($linestart !~ /^\s*$/) {
		$syntax_error = 1;
	}

	# Do the TAB-Completion

	if ($syntax_error) {

		foreach my $x (sort @nicks) {
			if($x =~ /^$word/i) {
				push(@$list, $x);
			}
		}
	}

	elsif ($counter == 0) {

		foreach my $x (sort (@switches_combo, @switches_nocombo, @nicks)) {
			if($x =~ /^$word/i) {
				push(@$list, $x);
			}
		}
	}

	elsif (($combo && $first_level_option && $second_level_option && $third_level_option) ||
	       ($combo && $first_level_option && !$second_level_option && !$third_level_option))
	{

		my @switches_second_level = grep !/^-sub$/, map { $_ = "-$_" }
					    keys %{ $switches{combo}{$first_level_option} };

		foreach my $x (sort (@switches_second_level, @switches_combo, @nicks)) {
			if($x =~ /^$word/i) {
				push(@$list, $x);
			}
		}
	}

	elsif ((!$combo && $counter == 1 && $first_level_option && $second_level_option && $third_level_option) ||
	       (!$combo && $counter == 1 && $first_level_option && !$second_level_option && !$third_level_option))
	{
		my @switches_second_level = grep !/^-sub$/, map { $_ = "-$_" }
					    keys %{ $switches{nocombo}{$first_level_option} };

		foreach my $x (sort (@switches_second_level)) {
			if($x =~ /^$word/i) {
				push(@$list, $x);
			}
		}
	}

	elsif ($combo && $first_level_option && $second_level_option && !$third_level_option) {
		my @switches_third_level = grep !/^\*$/,
					   keys %{ $switches{combo}{$first_level_option}{$second_level_option} };

		foreach my $x (sort (@switches_third_level)) {
			if($x =~ /^$word/i) {
				push(@$list, $x);
			}
		}
	}

	elsif (!$combo && $counter == 1 && $first_level_option && $second_level_option && !$third_level_option) {
		my @switches_third_level = grep !/^\*$/,
					   keys %{ $switches{nocombo}{$first_level_option}{$second_level_option} };

		foreach my $x (sort (@switches_third_level)) {
			if($x =~ /^$word/i) {
				push(@$list, $x);
			}
		}
	}

	Irssi::signal_stop();
}

sub signal_event_privmsg {
	my ($server, $data, $nick, $hostmask) = @_;
	my ($target, $text) = split / :/, $data, 2;
	my $target_lc = lc($target);
	my %channel;

	set_settings();

	return unless ($option{dau_remote_permissions} =~ /^1[01][01][01][01]$/);

	while ($option{dau_remote_channellist} =~ /\s*([^,\s]+)\s*,?/g) {
		my $item = $1;
		$item = lc($item);
		$channel{$item} = 1;
	}
	if (lc($option{dau_remote_channelpolicy}) eq 'allow') {
		return if ($channel{$target_lc});
	} elsif (lc($option{dau_remote_channelpolicy}) eq 'deny') {
		return unless ($channel{$target_lc});
	} else {
		return;
	}

	if ($text =~ /^hallo wie geht[\s?!1]*$/i) {
		set_settings();

		my $reply = $option{dau_remote_question_reply};
		$reply =~ s/(?<![\\])\$nick/$nick/g;
		$reply = parse_text($reply);

		output_text($server, $target, $reply);
	}
}

sub signal_nick_mode_changed {
	my ($channel, $nick, $setby, $mode, $type) = @_;
	my ($reply, %channel_perm);

	return unless ($channel->{server}->{nick} eq $nick->{nick});
	return if ($nick->{nick} eq $setby);
	return if ($setby eq 'irc.psychoid.net');

	set_settings();

	while ($option{dau_remote_channellist} =~ /\s*([^,\s]+)\s*,?/g) {
		my $item = $1;
		$item = lc($item);
		$channel_perm{$item} = 1;
	}
	if (lc($option{dau_remote_channelpolicy}) eq 'allow') {
		return if ($channel_perm{lc($channel->{name})});
	} elsif (lc($option{dau_remote_channelpolicy}) eq 'deny') {
		return unless ($channel_perm{lc($channel->{name})});
	} else {
		return;
	}

	if ($option{dau_remote_permissions} =~ /^[01]1[01][01][01]$/) {
		if ($mode eq '+' && $type eq '+') {
			$reply = $option{dau_remote_voice_reply};
			$reply =~ s/(?<![\\])\$nick/$setby/g;
			$reply = parse_text($reply);
		}
	}
	if ($option{dau_remote_permissions} =~ /^[01][01]1[01][01]$/) {
		if ($mode eq '@' && $type eq '+') {
			$reply = $option{dau_remote_op_reply};
			$reply =~ s/(?<![\\])\$nick/$setby/g;
			$reply = parse_text($reply);
		}
	}
	if ($option{dau_remote_permissions} =~ /^[01][01][01]1[01]$/) {
		if ($mode eq '+' && $type eq '-') {
			$reply = $option{dau_remote_devoice_reply};
			$reply =~ s/(?<![\\])\$nick/$setby/g;
			$reply = parse_text($reply);
		}
	}
	if ($option{dau_remote_permissions} =~ /^[01][01][01][01]1$/) {
		if ($mode eq '@' && $type eq '-') {
			$reply = $option{dau_remote_deop_reply};
			$reply =~ s/(?<![\\])\$nick/$setby/g;
			$reply = parse_text($reply);
		}
	}

	output_text($channel, $channel->{name}, $reply);
}

sub signal_send_text {
	my ($data, $server, $witem) = @_;
	my $output;

	return unless (defined($server) && $server && $server->{connected});
	return unless (defined($witem) && $witem &&
	              ($witem->{type} eq 'CHANNEL' || $witem->{type} eq 'QUERY'));

	if ($option{daumode_active}{$server->{tag}}{$witem->{name}} == 1) {
		if ($option{daumode_active_fix}{$server->{tag}}{$witem->{name}} eq '') {
			$output = parse_text($option{daumode_active_fix}{$server->{tag}}{$witem->{name}} . $data);
		} else {
			$output = parse_text($option{daumode_active_fix}{$server->{tag}}{$witem->{name}} . ' ' . $data);
		}

		output_text($witem, $witem->{name}, $output);

		Irssi::signal_stop();
	}
}

################################################################################
# Subroutines (statusbar)
################################################################################

sub statusbar_daumode {
	my ($item, $get_size_only) = @_;
	my $daumode;

	my $witem = Irssi::active_win()->{active};
	my $server = Irssi::active_server();

	my $theme = Irssi::current_theme();
	my $format = $theme->format_expand('{sb_daumode}');
	my $string;
	my $string_fix;

	if (defined($witem) && $witem &&
	    defined($server) && $server &&
	   ($witem->{type} eq 'CHANNEL' || $witem->{type} eq 'QUERY'))
	{

		if ($option{daumode_active}{$server->{tag}}{$witem->{name}} == 1) {
			$daumode = 'ON';
		} else {
			$daumode = 'OFF';
		}

		if ($format) {

			my $daumode_active_fix = $option{daumode_active_fix}{$server->{tag}}{$witem->{name}};
			$string_fix = $theme->format_expand("{sb_daumode_fix $daumode_active_fix}",
		                                            Irssi::EXPAND_FLAG_IGNORE_EMPTY());

			if ($daumode_active_fix ne '') {
				$string .= $string_fix;
			} else {
				$string = '';
			}

			$format = $theme->format_expand("{sb_daumode $daumode $string}");

		} else {

			if ($option{daumode_active_fix}{$server->{tag}}{$witem->{name}} ne '') {
				$format = "{sb daumode \%9$daumode\%9 ($option{daumode_active_fix}{$server->{tag}}{$witem->{name}})}";
			} else {
				$format = "{sb daumode \%9$daumode\%9}";
			}
		}

	} else {

		$item->{min_size} = $item->{max_size} = 0;
		return;
	}

	$item->default_handler($get_size_only, $format, undef, 1);
}

################################################################################
# Helper subroutines
################################################################################

sub def_dau_figlet_fontpath {
	my $figlet = `figlet -I2`;
	chomp($figlet);
	return $figlet;
}

sub def_dau_figlet_path {
	my $figlet = `which figlet`;
	chomp($figlet);
	return $figlet;
}

sub fix {
	my $string = shift;
	$string =~ s/^\t+//gm;
	return $string;
}

sub output_text {
	my ($thing, $target, $text) = @_;

	foreach my $line (split /\n/, $text) {
		if ($option{'print'}) {
			print CLIENTCRAP $line;
		} else {
			$thing->command("$option{command} $target $line");
		}
	}
}

sub parse_text {
	my $data = shift;
	my $output;
	%{ $option{queue} } = ();
	$option{command}    = 'MSG';
	$option{counter}    = 0;
	$option{daumode}    = 0;
	$option{'print'}    = 0;

	set_settings();

	OUTER: while ($data =~ /^--(\w+) ?/g) {

		my $first_level_option  = $1;

		# If its the first time we are in the OUTER loop, check
		# if the First-Level-Option is one of the few options,
		# which must not be combined.

		if (ref($switches{nocombo}{$first_level_option}{'sub'}) && $option{counter} == 0) {

			$data =~ s/^--\w+ ?//;

			# found a First-Level-Option

			$option{queue}{$option{counter}}{$first_level_option} = { };

			# Check for Second-Level-Options and Third-Level-Options.
			# Get all of them and put theme in the
			# $option{queue} hash

			while ($data =~ /^-(\w+) ('?)((?{ $2 })[^']+'|\S+) ?/g) {

				my $second_level_option  = $1;
				my $third_level_option   = $3;

				$third_level_option =~ s/'$// if ($2);

				# If $switches{nocombo}{$first_level_option}{$second_level_option}{'*'}:
				# The user can give any third_level_option on the commandline

				my $any_option =
				$switches{nocombo}{$first_level_option}{$second_level_option}{'*'} ? 1 : 0;

				if ($switches{nocombo}{$first_level_option}{$second_level_option}{$third_level_option} ||
				    $any_option)
				{
					$option{queue}{$option{counter}}{$first_level_option}{$second_level_option} = $third_level_option;
				}

				$data =~ s/^-(\w+) ('?)((?{ $2 })[^']+'|\S+) ?//;
			}

			# All done. Run the subroutine
			$output = &{ $switches{nocombo}{$first_level_option}{'sub'} }($data);

			return $output;
		}

		# Check for all those options that can be combined.

		elsif (ref($switches{combo}{$first_level_option}{'sub'})) {

			$data =~ s/^--\w+ ?//;

			# found a First-Level-Option

			$option{queue}{$option{counter}}{$first_level_option} = { };

			# Check for Second-Level-Options and
			# Third-Level-Options. Get all of them and put them
			# in the $option{queue} hash

			while ($data =~ /^-(\w+) ('?)((?{ $2 })[^']+'|\S+) ?/g) {

				my $second_level_option  = $1;
				my $third_level_option   = $3;

				$third_level_option =~ s/'$// if ($2);

				# If $switches{combo}{$first_level_option}{$second_level_option}{'*'}:
				# The user can give any third_level_option on the commandline

				my $any_option =
				$switches{combo}{$first_level_option}{$second_level_option}{'*'} ? 1 : 0;

				# known option => Put it in the hash

				if ($switches{combo}{$first_level_option}{$second_level_option}{$third_level_option}
			            || $any_option)
				{
					$option{queue}{$option{counter}}{$first_level_option}{$second_level_option} = $third_level_option;
					$data =~ s/^-(\w+) ('?)((?{ $2 })[^']+'|\S+) ?//;
				} else {
					last OUTER;
				}
			}
		}
	} continue {
		$option{counter}++;
	}

	# text to subroutines

	$output = $data;

	# If theres no text left over, take one item of dau_random_messages

	my @strings;
	if ($output =~ /^\s*$/) {
		while ($option{dau_standard_messages} =~ /\s*([^,]+)\s*,?/g) {
			my $item = $1;
			push @strings, $item;
		}
		if (@strings > 0) {
			$output = $strings[rand(@strings)];
		} else {
			$output = '';
		}
	}

	# No options? Get options from setting dau_standard_options and run
	# parse_text() again

	if (keys %{ $option{queue} } == 0) {

		if (!$option{subcounter}) {
			$option{subcounter}++;
			$output = parse_text("$option{dau_standard_options} $output");
		} else {
			print_err('Invalid value for setting dau_standard_options. Will use %9--moron%9 instead!');
			$output =~ s/^\Q$option{dau_standard_options}\E //;
			$output = parse_text("--moron $output");
		}

	} else {

		$option{counter} = 0;

		foreach (keys %{ $option{queue} }) {

			my ($first_level_option) = keys %{ $option{queue}{$option{counter}} };

			if ($switches{combo}{$first_level_option}) {
				$output = &{ $switches{combo}{$first_level_option}{'sub'} }($output);
			}
			$option{counter}++;
		}
	}

	# reset subcounter

	$option{subcounter} = 0;

	# return text

	return $output;
}

sub print_err {
	my $text = shift;

	print CLIENTCRAP '%Rdau.pl error%n: ' . $text;
}

sub print_out {
	my $text = shift;

	print CLIENTCRAP '%9dau.pl%9: ' . $text;
}

sub set_settings {
	# setting changed/added => change/add it here

	# Integer

	# boolean
	$option{dau_figlet_print_font} = Irssi::settings_get_bool('dau_figlet_print_font');

	# String
	$option{dau_delimiter_string}                = Irssi::settings_get_str('dau_delimiter_string');
	$option{dau_figlet_fontlist}                 = Irssi::settings_get_str('dau_figlet_fontlist');
	$option{dau_figlet_fontpath}                 = Irssi::settings_get_str('dau_figlet_fontpath');
	$option{dau_figlet_fontpolicy}               = Irssi::settings_get_str('dau_figlet_fontpolicy');
	$option{dau_figlet_path}                     = Irssi::settings_get_str('dau_figlet_path');
	$option{dau_moron_eol_style}                 = Irssi::settings_get_str('dau_moron_eol_style');
	$option{dau_moron_substitutions_permissions} = Irssi::settings_get_str('dau_moron_substitutions_permissions');
	$option{dau_random_options}                  = Irssi::settings_get_str('dau_random_options');
	$option{dau_remote_channellist}              = Irssi::settings_get_str('dau_remote_channellist');
	$option{dau_remote_channelpolicy}            = Irssi::settings_get_str('dau_remote_channelpolicy');
	$option{dau_remote_deop_reply}               = Irssi::settings_get_str('dau_remote_deop_reply');
	$option{dau_remote_devoice_reply}            = Irssi::settings_get_str('dau_remote_devoice_reply');
	$option{dau_remote_op_reply}                 = Irssi::settings_get_str('dau_remote_op_reply');
	$option{dau_remote_permissions}              = Irssi::settings_get_str('dau_remote_permissions');
	$option{dau_remote_question_reply}           = Irssi::settings_get_str('dau_remote_question_reply');
	$option{dau_remote_voice_reply}              = Irssi::settings_get_str('dau_remote_voice_reply');
	$option{dau_standard_messages}               = Irssi::settings_get_str('dau_standard_messages');
	$option{dau_standard_options}                = Irssi::settings_get_str('dau_standard_options');
	$option{dau_words_range}                     = Irssi::settings_get_str('dau_words_range');
}
