use strict;

use vars qw($VERSION %IRSSI);
$VERSION = "2002120501";
%IRSSI = (
    authors     => "Stefan 'tommie' Tomanek",
    contact     => "stefan\@pico.ruhr.de",
    name        => "multipaste",
    description => "Helps pasting multiple lines to a channel",
    license     => "GPLv2",
    url         => "",
    changed     => "$VERSION",
    modules     => ""
);

use Irssi 20020324;
use vars qw(%item);

sub sig_send_text ($$$) {
    my ($line, $server, $witem) = @_;
    return unless (Irssi::settings_get_bool('multipaste_auto'));
    return unless (ref $server);
    return unless ($witem && ($witem->{type} eq 'CHANNEL' || $witem->{type} eq 'QUERY'));
    $line =~ s/\t/    /g;
    if ($witem->{name} eq $item{channel} && $server->{tag} eq $item{server}) {
	Irssi::timeout_remove($item{timeout});
	#Irssi::command("BIND -delete tab");
	my $timeout = 10;
	chomp($line);
	$item{text} .= $line."\n";
	$item{timeout} = Irssi::timeout_add($timeout, \&send_item, undef);
	Irssi::signal_stop();
    } else {
	unless ($line eq '') {
	    Irssi::signal_stop();
	    cmd_paste($line, $server, $witem);
	}
    }
}

sub sig_send_command ($$$) {
    my ($line, $server, $witem) = @_;
    return unless (Irssi::settings_get_bool('multipaste_auto'));
    return unless (ref $witem && ($witem->{type} eq 'CHANNEL' || $witem->{type} eq 'QUERY'));
    # This does not work when the first line starts with 
    return unless $item{text};
    $line =~ s/\t/    /g;
    if ($witem->{name} eq $item{channel} && $server->{tag} eq $item{server}) {
        Irssi::timeout_remove($item{timeout});
	#Irssi::command("BIND -delete tab");
        my $timeout = 10;
        chomp($line);
        $item{text} .= $line."\n";
        $item{timeout} = Irssi::timeout_add($timeout, \&send_item, undef);
        Irssi::signal_stop();
    } else {
        Irssi::signal_stop();
        cmd_paste($line, $server, $witem);
    }
}


sub send_item {
    my $server = Irssi::server_find_tag($item{server});
    my $channel = $server->window_item_find($item{channel});
    my $prefix = Irssi::settings_get_str('multipaste_prefix');
    $prefix = $item{prefix}.': '.$prefix if $item{prefix};
    if (scalar( split(/\n/, $item{text}) ) > 1) {
	#Irssi::command("BIND tab word_completion");
	foreach (split(/\n/, $item{text})) {
	    $server->command('MSG -- '.$channel->{name}.' '.$prefix.$_);
	}
    } else {
	my $text = join("", split(/\n/, $item{text}));
	my $prefix = $item{prefix} ? $item{prefix}.': ' : '';
	unless ($prefix.$text eq "\n") {
	    $server->command('MSG -- '.$channel->{name}.' '.$prefix.$text);
	}
    }
    Irssi::timeout_remove($item{timeout});
    %item = ();
}

sub cmd_paste ($$$) {
    my ($args, $server, $witem) = @_;
    return unless ref $witem;
    return if (%item);
    chomp($args);
    my $timeout = 10;
    if ($args =~ /^(.+?): (.*)/ && $witem->{type} eq 'CHANNEL' && $witem->nick_find($1)) {
	$item{prefix} = $1;
	$item{text} .= $2."\n";
    } else {
	$item{text} .= $args."\n";
    }
    $item{server} = $server->{tag};
    $item{channel} = $witem->{name};
    $item{timeout} = Irssi::timeout_add($timeout, \&send_item, undef);
}

sub sig_word_complete ($$$$$) {
    my ($list, $window, $word, $linestart, $want_space) = @_;
    if (scalar( split(/\n/, $item{text}) ) >= 1) {
	push @$list, $linestart.$word.'    ';
	Irssi::signal_stop();
    }
}

Irssi::settings_add_bool($IRSSI{name}, 'multipaste_auto', 1);
Irssi::settings_add_str($IRSSI{name}, 'multipaste_prefix', '|');
Irssi::command_bind('multipaste', \&cmd_paste);
Irssi::signal_add('send text', 'sig_send_text');
Irssi::signal_add('send command', 'sig_send_command');
Irssi::signal_add_first('complete word', 'sig_word_complete');

print CLIENTCRAP "%B>>%n ".$IRSSI{name}." ".$VERSION." loaded";
