# Fixes for multiple servers and window items by dg

use Irssi;
use vars qw($VERSION %IRSSI);

$VERSION = '0.1';
%IRSSI = (
    authors     => 'Kimmo Lehto',
    contact     => 'kimmo@a-men.org' ,
    name        => 'Timer',
    description => 'Provides /timer command for mIRC/BitchX type timer functionality.',
    license     => 'Public Domain',
    changed	=> 'Wed Mar 27 14:51 EET 2002'
);

sub timer_command
{
	my ($name) = @_;
	
	if ($_timer_numtimes{$name} ne "Infinite")
	{
		if($_timer_numtimes{$name}-- < 1)
		{
			cmd_timerstop($name);
			return;
		}
	}

	if (!$_timer_witem{$name})
	{
		Irssi::command("$_timer_command{$name}");
	}
	else
	{
		$_timer_witem{$name}->command("$_timer_command{$name}");
	}
	return;
}
sub cmd_timerlist
{
	Irssi::print("Active timers:");
	my ($name, $numtimes);
        foreach $name (keys %_timer)  
        {
		if ($_timer_numtimes{$name} ne "Infinite") { $numtimes = "($_timer_numtimes{$name} repeats left)"; }
              	Irssi::print("$name = $_timer_command{$name} $numtimes");
		$numtimes = undef;
        }
	Irssi::print("End of /timer list");
}

sub cmd_timerstop
{
	my ($name) = @_;
        if ($_timer{$name})
        {
              Irssi::timeout_remove($_timer{$name});
              delete $_timer{$name};
              delete $_timer_command{$name};
	      delete $_timer_witem{$name};
	      delete $_timer_numtimes{$name};
              Irssi::print("Timer \"$name\" stopped.");
        }
        else
        {
              Irssi::print("No such timer \"$name\".");
        }
}

sub cmd_timerstart
{
	my ($name, $interval, $times, $command,$witem) = @_;

	if ($_timer{$name}) 
	{
		Irssi::print("Timer \"$name\" already active.");
	}
	else
	{
		$_timer_numtimes{$name} = $times;
		if ($times eq "Infinite") { $times = 'until stopped.'; } else { $times .= " times."; }
		$_timer_command{$name} = $command;
		$_timer_witem{$name} = $witem;
		Irssi::print("Starting timer \"$name\" repeating \"$command\" every $interval seconds $times");
		$_timer{$name} = Irssi::timeout_add($interval * 1000, 'timer_command', $name);
	}
}
	
sub cmd_timer
{
	my ($arg, $server, $witem) = @_;
	if ($arg eq "list") { cmd_timerlist(); }
	elsif ($arg =~ /^stop\W+(.*)$/) { cmd_timerstop($1); }
	elsif ($arg =~ /^add\W+(\w+)\W+(\d+)\W+(\d+)\W+(.*)$/)
        {
                cmd_timerstart($1, $2, $3, $4,$witem);
        }
	elsif ($arg =~ /^add\W+(\w+)\W+(\d+)\W+(.*)$/) 
	{
		cmd_timerstart($1, $2, "Infinite", $3,$witem);
	}
	else
	{
                Irssi::print("Timer - Usage: /timer add <name> <interval> [repeat] <command>");
                Irssi::print(" - Repeat <command> every <interval> seconds [repeat] times (or until stopped) and call it " . 
                             "<name> so you can /timer stop <name> it. You can /timer list them.");
	}
}
	
Irssi::command_bind('timer', 'cmd_timer');


